#include "stdafx.h"
#include "tranchar.h"


unsigned int hankaku2zen(int hankaku)
{
    static unsigned int z[64] = {
        0x2121,0x2123,0x2156,0x2157,0x2122,0x2126,0x2572,0x2521,
        0x2523,0x2525,0x2527,0x2529,0x2563,0x2565,0x2567,0x2543,
        0x213c,0x2522,0x2524,0x2526,0x2528,0x252a,0x252b,0x252d,
        0x252f,0x2531,0x2533,0x2535,0x2537,0x2539,0x253b,0x253d,
        0x253f,0x2541,0x2544,0x2546,0x2548,0x254a,0x254b,0x254c,
        0x254d,0x254e,0x254f,0x2552,0x2555,0x2558,0x255b,0x255e,
        0x255f,0x2560,0x2561,0x2562,0x2564,0x2566,0x2568,0x2569,
        0x256a,0x256b,0x256c,0x256d,0x256f,0x2573,0x212b,0x212c };

    if (hankaku < 0xa0 || hankaku > 0xdf) return 0;
    return z[hankaku - 0xa0];
}

// SJISR[hEUCR[hɕϊ
unsigned int CTransChar::sjis2euc(unsigned int sjis)
{
	unsigned int hib, lob;
          
    hib = (sjis >> 8) & 0xff;
	lob = sjis & 0xff;
	hib -= (hib <= 0x9f) ? 0x71 : 0xb1;
	hib = (hib << 1) + 1;
	if (lob >= 0x9e) 
	{
		lob -= 0x7e;
		hib++;
	} 
	else if (lob > 0x7f) lob -= 0x20;
	else lob -= 0x1f;

	hib |= 0x80;
	lob |= 0x80;

	return (hib << 8) | lob;
}

// EUCR[hSJISR[hɕϊ
unsigned int CTransChar::euc2sjis(unsigned int euc)
{	
	unsigned int jis;
	unsigned int hib, lob;

	// pJȉ
	if ((euc & 0xff00) == 0x8e00)
		jis = hankaku2zen(euc & 0xff);
	else jis = euc & ~0x8080;
 
	hib = (jis >> 8) & 0xff;
	lob = jis & 0xff;
	lob += (hib & 1) ? 0x1f : 0x7d;
        
	if (lob >= 0x7f) lob++;
    hib = ((hib - 0x21) >> 1) + 0x81;
        
	if (hib > 0x9f) hib += 0x40;
   
	return (hib << 8) | lob;
      
}

// EUC̕SJIS̕ɕϊ
CString CTransChar::euc2sjis(CString eucCString)
{
	CString sjisCString;

	// CStringɊi[{ꃁbZ[W𕶎ƂĎo
	const char *p = (LPCTSTR)eucCString;

	// 񒷂o
	int eucCStringLength = eucCString.GetLength();

	for(int i = 0; i < eucCStringLength; i++)
	{
		BYTE dummyByte = p[i];
		
		// ASCIIR[hȂ
		if ((dummyByte >= 0x00) & (dummyByte <= 0x7f))
		{
			sjisCString += dummyByte;
		}
		// pȂ
		else if (dummyByte == 0x8e)
		{
			BYTE dummyByte1 = p[i+1];
			sjisCString += dummyByte1;
			i++;
		}
		// A⏕Ȃ
		else if (dummyByte == 0x8f)
		{
			// later;
			sjisCString += dummyByte;
		}
		else 
		{
			unsigned int KanjiEucCode = 0;
			
			KanjiEucCode = 0x00000000;
			KanjiEucCode = (((0x000000ff & p[i]) <<8) | (0x000000ff & p[i+1]));
			
			short KanjiSjisCode = (short)theTransChar.euc2sjis(KanjiEucCode);		

			char kanji[3];
			kanji[0] = (KanjiSjisCode & 0xff00) >> 8;
			kanji[1] = (KanjiSjisCode & 0x00ff) ;
			kanji[2] = 0x00 ;
	
			sjisCString  += kanji; // 
			
			i++;
		}

	}
	
	return sjisCString;
}

// SJIS̕EUC̕ɕϊ
CString CTransChar::sjis2euc(CString sjisCString)
{
	CString eucCString;

	// CStringɊi[{ꃁbZ[W𕶎ƂĎo
	const char *p = (LPCTSTR)sjisCString;

	// 񒷂o
	int sjisCStringLength = sjisCString.GetLength();

	for(int i = 0; i < sjisCStringLength; i++)
	{
		BYTE dummyByte = p[i];
		
		// ASCIIR[hȂA
		if ((dummyByte >= 0x00) && (dummyByte <= 0x7f))	
		{
			eucCString+= dummyByte;
		}
		// JISpJiȂ
		else if ((dummyByte >= 0xa0) && (dummyByte <= 0xdf))	
		{
			BYTE dummyByte1 = 0x8e;
			eucCString += dummyByte1;
			eucCString += dummyByte;
		}
		// JISȂ
		else if (((dummyByte >= 0x81) && (dummyByte <= 0x9f)) 
			|| ((dummyByte >= 0xe0) && (dummyByte <= 0xef)))
		{
			// ܂A{ꕶSJISR[hƂđgݗĂ(Ɨǂ͂͂j
			unsigned int hib = 0;
			unsigned int lob = 0;
			unsigned int KanjiSjisCode = 0;

			hib = ( p[i] & 0x000000ff) << 8;
			lob = p[i + 1] & 0x000000ff;
			KanjiSjisCode = ( hib | lob ) & 0x0000ffff ;

			// SJIS->EUCR[hϊ
			short KanjiEucCode = (short)theTransChar.sjis2euc(KanjiSjisCode);

			char kanji[3];
			kanji[0] = (KanjiEucCode & 0xff00) >> 8;
			kanji[1] = (KanjiEucCode & 0x00ff) ;
			kanji[2] = 0x00;

			eucCString += kanji;
			const char *dummyp = (LPCTSTR)eucCString;

			i++;
		}
	}
		

	return eucCString;
}

// SJIS̊R[h܂܂Ă邩ǂ𔻒肷
bool CTransChar::isSjisKanji(CString sjisCString)
{
	// CStringɊi[{ꃁbZ[W𕶎ƂĎo
	const char *p = (LPCTSTR)sjisCString;

	// 񒷂o
	int sjisCStringLength = sjisCString.GetLength();

	for(int i = 0; i < sjisCStringLength; i++)
	{
		BYTE dummyByte = p[i];
		
		if
			(
				((dummyByte >= 0x81) && (dummyByte <= 0x9f)) // JIS 
			||  ((dummyByte >= 0xe0) && (dummyByte <= 0xef)) // JIS 
			||  ((dummyByte >= 0xa0) && (dummyByte <= 0xdf)) // ݶJi
			)
		{
			return true;
		}
	}
	
	return false;
}


// sR[hB
CString CTransChar::eraseJLineFeed(CString sjisCString)
{
	CString tempCString;
	tempCString.Empty();

	// CStringɊi[{ꃁbZ[W𕶎ƂĎo
	const char *p = (LPCTSTR)sjisCString;

	// 񒷂o
	int sjisCStringLength = sjisCString.GetLength();

	for(int i = 0; i < sjisCStringLength; i++)
	{
		BYTE dummyByte1 = p[i];
		BYTE dummyByte2 = p[i+1];

		if ((dummyByte1 == 0x0d) && (dummyByte2 == 0x0a))
			i++;
		else
			tempCString += p[i];
	}

	return tempCString;
}


// sR[h󔒂ɕύXB
CString CTransChar::eraseELineFeed(CString sjisCString)
{
	CString tempCString;
	tempCString.Empty();

	// CStringɊi[{ꃁbZ[W𕶎ƂĎo
	const char *p = (LPCTSTR)sjisCString;

	// 񒷂o
	int sjisCStringLength = sjisCString.GetLength();

	for(int i = 0; i < sjisCStringLength; i++)
	{
		BYTE dummyByte1 = p[i];
		BYTE dummyByte2 = p[i+1];

		if ((dummyByte1 == 0x0d) && (dummyByte2 == 0x0a))
		{
			tempCString += 0x20;
			i++;
		}
		else if (dummyByte1 == '.')
			tempCString += ". ";
		else
			tempCString += p[i];
	}

	return tempCString;
}
